#define HTTP_SERVER_SOFTWARE		"nhttpd"

#define HTTP_METHOD_GET			0
#define HTTP_METHOD_HEAD		1
#define HTTP_METHOD_POST		2

#define HTTP_PROTOCOL_09		"HTTP/0.9"
#define HTTP_PROTOCOL_10		"HTTP/1.0"
#define HTTP_PROTOCOL_11		"HTTP/1.1"

#define HTTP_DECODED_URL_SIZE		1024*2
#define HTTP_FILENAME_SIZE		1024*2
#define HTTP_RESPONSE_HEADER_SIZE	1024*2

#define HTTP_RESPONSE_CONTENT_LENGTH	0x001
#define HTTP_RESPONSE_CONTENT_TYPE	0x002
#define HTTP_RESPONSE_LAST_MODIFIED	0x004
#define HTTP_RESPONSE_KEEP_ALIVE	0x008

#define HTTP_RESPONSE_STAT_200		"OK"
#define HTTP_RESPONSE_STAT_304		"Not Modified"
#define HTTP_RESPONSE_STAT_400		"Bad Request"
#define HTTP_RESPONSE_STAT_401		"Unauthorized"
#define HTTP_RESPONSE_STAT_403		"Forbidden"
#define HTTP_RESPONSE_STAT_404		"Not Found"

/* HTTP û  ü */
struct http_request_header_t {
	short		method;
	char		*encoded_url;
	char		decoded_url[HTTP_DECODED_URL_SIZE];
	char		*query;
	char		filename[HTTP_FILENAME_SIZE];
	short		protocol;

	char		*host;
	int		host_len;
	int		port;

	char		*accept_charset;
	char		*accept_language;
	char		*accept_ranges;
	int		content_length;
	char		*connection;
	char		*accept;
	char		*accept_encoding;
	time_t		if_modified_since;
	char		*referer;
	char		*user_agent;

	short		static_file;
};

/* HTTP  ü */
struct http_response_header_t {
	short		status;

	int		content_length;
	char		*content_type;
	char		last_modified[64];
	int		keep_alive;
	short		connection;		/* 0: close,  0 > keep-alive */

	int		flags;
};


short http_request_end( char *buf, int buf_len );
int http_request_preparse( struct http_request_header_t *header, char *buf, int buf_len );
int http_request_parse( struct http_request_header_t *header, char *buf, int buf_len, int offset );
short http_keep_alive( struct http_request_header_t *request_header, struct http_response_header_t *response_header, int timeout );
void http_last_modified( struct http_response_header_t *response_header, time_t mktime );
void http_content_length( struct http_response_header_t *response_header, int content_length );
void http_content_type( struct http_request_header_t *request_header, struct http_response_header_t *response_header );

int http_response_send( struct http_response_header_t *header, short protocol, int sockfd );
char *http_status_code( short status );
char *http_protocol( short protocol );

void http_make_date( time_t now );
int http_request_parse_accept_charset( struct http_request_header_t *header, char *buf );
int http_request_parse_accept_language( struct http_request_header_t *header, char *buf );
int http_request_parse_accept_ranges( struct http_request_header_t *header, char *buf );
int http_request_parse_content_length( struct http_request_header_t *header, char *buf );
int http_request_parse_connection( struct http_request_header_t *header, char *buf );
int http_request_parse_accept( struct http_request_header_t *header, char *buf );
int http_request_parse_accept_encoding( struct http_request_header_t *header, char *buf );
int http_request_parse_if_modified_since( struct http_request_header_t *header, char *buf );
int http_request_parse_referer( struct http_request_header_t *header, char *buf );
int http_request_parse_user_agent( struct http_request_header_t *header, char *buf );
int http_request_parse_host( struct http_request_header_t *header, char *buf );

time_t http_asctime2time_t( char *asctime );
int http_ascmon2int( char *ascmonth );
void http_decode_url( char *to, char *from );
void http_de_dotdot( char *file );
int http_hexit( char c );

