#include <header.h>

/* node search by node tagname */
xmlNode *xml_find_next_by_tag( xmlNode *start, char *tag_name )
{
	xmlNode *next;

	if ( start == NULL )
		return NULL;

	next = start->next;

	while ( next != NULL )
	{
		if ( next->name != NULL && strcasecmp( next->name, tag_name ) == 0 )
		{
			return next;
			break;
		}
		next = next->next;
	}

	return NULL;
}

xmlNode *xml_find_child_by_tag( xmlNode *parent, char *tag_name )
{
	return xml_find_next_by_tag( parent->children, tag_name );
}

xmlNode *xml_find_doc_child_by_tag( xmlDocPtr parent, char *tag_name )
{
	return xml_find_next_by_tag( parent->children, tag_name );
}


/* return values */
int xml_get_config_int_value( xmlNode *parent, char *tag_name, int default_value )
{
	xmlNode	*node;

	node = xml_find_child_by_tag( parent, tag_name );
	return ( node == NULL || node->children == NULL ) ? default_value : atoi( XML_GET_CONTENT(node->children) );
}

char *xml_get_config_char_value( xmlNode *parent, char *tag_name, char *default_value )
{
	xmlNode	*node;

	if ( parent != NULL && tag_name == NULL )
		return (char *) XML_GET_CONTENT(parent->children);

	node = xml_find_child_by_tag( parent, tag_name );
	return ( node == NULL || node->children == NULL ) ? default_value : (char *) XML_GET_CONTENT(node->children);
}

unsigned short xml_get_config_boolean_value( xmlNode *parent, char *tag_name, unsigned short default_value )
{
	xmlNode	*node;
	char *text;

	node = xml_find_child_by_tag( parent, tag_name );
	if ( node == NULL || node->children == NULL )
		return default_value;

	text = (char *) XML_GET_CONTENT(node->children);
	if ( strncmp(text, "1", 1) == 0 )
		return 1;
	else if ( strncmp(text, "0", 1) == 0 )
		return 0;

	return ((strncasecmp(text,"true", 4) == 0)?(unsigned short) 1:(unsigned short) 0);
}


xmlChar *xml_get_attr_to_char( xmlNode *node, xmlChar *attr_name )
{
	return xmlGetNoNsProp( node, attr_name );
}

int xml_get_attr_to_int( xmlNode node, xmlChar *attr_name, int def_value )
{
	xmlChar *ret;
	ret = xmlGetNoNsProp( &node, attr_name );
	if ( ret == NULL )
		return def_value;
	else
		return atoi(ret);
}

double xml_get_attr_to_double( xmlNode node, xmlChar *attr_name, double def_value )
{
	xmlChar *ret;
	ret = xmlGetNoNsProp( &node, attr_name );
	if ( ret == NULL )
		return def_value;
	else
		return atof(ret);
}

