#include <header.h>

int set_mime( struct mime_t **mimes, int *mimes_len )
{
	int			file_fd;
	FILE			*fd;

	extern struct config_t	config;

	struct stat	filestat;
	char		*content_type;
	char		*static_content;
	int		statics;
	char		ext[1024];
	int		line_len, line_count, len;
	int		charset_len;

	file_fd = open_real_file( config.file_mime, &filestat );
	if ( file_fd < 0 )
	{
		fprintf(stderr, "%s mime file not found.\n", config.file_mime);
		return -1;
	}

	fd = fdopen( file_fd, "r" );
	if ( fd == NULL )
	{
		fprintf(stderr, "%s mime file not found.\n", config.file_mime);
		close( file_fd );
		return -1;
	}

	line_count = 0;
	while ( fgets( ext, 1024, fd ) )
	{
		if ( ext[0] == '\n' || ext[0] == '\r' || ext[0] == '#' )
			continue;
		line_count++;
	}

	if ( line_count <= 0 )
	{
		fclose( fd );
		close( file_fd );
		return 0;
	}

	*mimes = malloc( sizeof(struct mime_t) * line_count );
	bzero( *mimes, sizeof(struct mime_t) * line_count );

	fseek( fd, 0, SEEK_SET );

	charset_len = strlen( config.default_charset );

	line_count = 0;
        while ( fgets( ext, 1024, fd ) )
        {
		if ( ext[0] == '\n' || ext[0] == '\r' || ext[0] == '#' )
			continue;

		line_len = strlen( ext );
		if ( ext[line_len - 1] == '\r' || ext[line_len - 1] == '\n' )
		{
			ext[line_len - 1] = '\0';
			line_len--;
		}

		if ( ext[line_len - 1] == '\r' || ext[line_len - 1] == '\n' )
		{
			ext[line_len - 1] = '\0';
			line_len--;
		}

		content_type = index( ext, '\t' );
		if ( content_type == (char *) 0 )
			continue;
		content_type[0] = '\0';
		content_type++;
		content_type += strspn( content_type, " \t" );

		static_content = index( content_type, '\t' );
		if ( static_content == (char *) 0 )
			statics = 1;
		else
		{
			static_content[0] = '\0';
			static_content++;
			statics = atoi( static_content );
		}

		len = strlen( ext );
		(*mimes)[line_count].ext_len = len;
		(*mimes)[line_count].ext = malloc( len + 1 );
		bzero( (*mimes)[line_count].ext, len + 1 );
		strncpy( (*mimes)[line_count].ext, ext, len);

		len = strlen( content_type );
		if ( strstr( content_type, "charset" ) != (char *) 0 )
		{
			(*mimes)[line_count].content_type = malloc( len + charset_len );
			bzero( (*mimes)[line_count].content_type, len + charset_len );
			snprintf( (*mimes)[line_count].content_type, ( len + charset_len ), content_type, config.default_charset );
		}
		else
		{
			(*mimes)[line_count].content_type = malloc( len + 1 );
			bzero( (*mimes)[line_count].content_type, len + 1 );
			strncpy( (*mimes)[line_count].content_type, content_type, len);
		}

		(*mimes)[line_count].static_content = statics;

		line_count++;
	}

	*mimes_len = line_count;

	fclose( fd );
	close( file_fd );

	return 1;
}


#define	_MIME_DEFAULT	"text/plain"

char *mime_type( struct http_request_header_t *header )
{
	char			*ext;
	int			len, i;

	extern int		mimes_len;
	extern struct mime_t	*mimes;

	if ( mimes_len <= 0 )
		return _MIME_DEFAULT;

	ext = rindex( header->filename, '.' );
	if ( ext == (char *) 0 )
		return _MIME_DEFAULT;

	ext++;
	len = strlen(ext);

	for ( i=0; i < mimes_len; i++ )
	{
		if ( mimes[i].ext_len == len && strncasecmp( mimes[i].ext, ext, len ) == 0 )
		{
			header->static_file = mimes[i].static_content;
			return mimes[i].content_type;
		}
	}

	return _MIME_DEFAULT;
}
