#include <header.h>

// 
void *proc(void *data)
{
#ifdef __NONBLOCK
	int				timeout;
	struct pollfd			listen_poll;
	int				poll_res;
#endif

	struct thread_t			*self = ( (struct thread_t *) data );
	int				read_len;
	int				read_res;
	char				*buf;
	int				buf_len;

	short				keep_alive;

	struct http_request_header_t	request_header;
	int				preparse_len;
	struct http_response_header_t	response_header;

	struct stat			filestat;
	int				file_fd;

	struct vhost_t			*vhost;

	extern struct config_t		config;
	extern int			terminate;


	buf_len = config.default_buffer_size;

	buf = malloc( buf_len );
	if ( buf == (char *)0 )
	{
		self->status = STAT_TERMINATED;
		pthread_exit(0);
	}

#ifdef __NONBLOCK
	timeout = config.timeout * 1000;
	listen_poll.events = (POLLIN|POLLPRI|POLLERR|POLLHUP);
#endif

	keep_alive = FALSE;

	while ( 1 )
	{
thread_loop_start:
		/*  ü/ ʱȭ */
		memset( buf, 0, buf_len );
		memset( &request_header, 0, sizeof(struct http_request_header_t) );
		memset( &response_header, 0, sizeof(struct http_response_header_t) );
		memset( &filestat, 0, sizeof(struct stat) );

		vhost = NULL;
		read_len = 0;

		/* ⺻  200 ̴ */
		response_header.status = 200;

		request_header.port = config.port[0];

		/* keep_alive Ʈ, ⺻ FALSE ̴ */
		if ( keep_alive == FALSE )
		{
			self->sockfd = -1;

			self->status = STAT_SLEEPING;

			/*  ñ׳ ٸ */
			pthread_mutex_lock( &(self->mutex) );
			pthread_cond_wait( &(self->cond), &(self->mutex) );
			pthread_mutex_unlock( &(self->mutex) );
		}

		/*  ȣ */
		if ( terminate == TRUE )
			break;

		/* keep_alive FALSE ʱȭ */
		keep_alive = FALSE;

		/*  ߸ Ҵ Ǿ ó ư */
		if ( self->sockfd < 0 )
		{
			close( self->sockfd );
			continue;
		}

#ifdef __NONBLOCK
		/* pollfd  Ѵ */
		listen_poll.fd = self->sockfd;
#endif

		/* HTTP û ٸ  */
		do
		{
#ifdef __NONBLOCK
			/* poll ̿   */
			poll_res = poll( &listen_poll, 1, timeout );

			if ( poll_res == 0 )
			{
				/* ŸӾƿ̸  ư */
				close( self->sockfd );
				goto thread_loop_start;
			}
			else if ( listen_poll.revents & POLLIN )
			{
#endif
				/* û д´ */
				read_res = read( self->sockfd, &(buf[read_len]), buf_len-read_len );
				if ( read_res <= 0 )
				{
					/* ̰ų   */
					close( self->sockfd );
					goto thread_loop_start;
				}
				read_len += read_res;
#ifdef __NONBLOCK
			}
			else
			{
				/*    */
				close( self->sockfd );
				goto thread_loop_start;
			}
#endif
		}
		while ( http_request_end( buf, read_len ) != TRUE );
		/* HTTP û   д´ */

		//printf("%s\n",buf);

		/* HTTP û pre parse */
		preparse_len = http_request_preparse( &request_header, buf, read_len );

		/* û ߸ ߸ û(400 )  ´ */
		if ( preparse_len < 0 )
		{
			response_header.status = 400;	/* bad request */
			http_content_length( &response_header, 0 );
			http_response_send( &response_header, request_header.protocol, self->sockfd );

			close(self->sockfd);
			tlog_record( LOG_ERR, HTTP_RESPONSE_STAT_400, NULL, &request_header, &response_header );
			continue;
		}

		/* HTTP  1.0  :  Ľ */
		if ( request_header.protocol == 10 )
			http_request_parse( &request_header, buf, read_len, preparse_len );
		/* HTTP  1.1  :  Ľ, keep_alive  */
		else if ( request_header.protocol == 11 )
		{
			http_request_parse( &request_header, buf, read_len, preparse_len );
			keep_alive = http_keep_alive( &request_header, &response_header, config.timeout );
		}

		file_fd = get_vhost_filefd( &request_header, &filestat, vhost );

		if ( file_fd < 0 )
		{
			/*      ã (404 )  keep_alive   ų  */
			response_header.status = 404;
			http_content_length( &response_header, 0 );

			if ( http_response_send( &response_header, request_header.protocol, self->sockfd ) < 0 )
				keep_alive = FALSE;

			if ( keep_alive == FALSE )
				close(self->sockfd);
			tlog_record( LOG_ERR, HTTP_RESPONSE_STAT_404, vhost, &request_header, &response_header );
			continue;
		}

		/*  Žð Ͽ   (304)   Ȯ */
		if ( request_header.if_modified_since > 0 && filestat.st_mtime == request_header.if_modified_since )
		{
			request_header.method = HTTP_METHOD_HEAD;
			response_header.status = 304;
		}
		else
		{
			/*  Ǿ  ũ⸦ Ѵ */
			http_last_modified( &response_header, filestat.st_mtime );
			http_content_length( &response_header, (int) filestat.st_size );
			http_content_type( &request_header, &response_header );
		}

		/*    */
		if ( http_response_send( &response_header, request_header.protocol, self->sockfd ) < 0 )
		{
			close( self->sockfd );
			close( file_fd );
			keep_alive = FALSE;
			tlog_record( LOG_ERR, "Response header send error.", vhost, &request_header, &response_header );
			continue;
		}

		/*    ϴ Ȯ */
		if ( request_header.method != HTTP_METHOD_HEAD )
		{
			/*   о Ѵ */
			if ( sendfile_response( self->sockfd, file_fd, filestat.st_size ) < 0 )
			{
				close( self->sockfd );
				close( file_fd );
				keep_alive = FALSE;
				tlog_record( LOG_ERR, "Sendfile Error.", vhost, &request_header, &response_header );
				continue;
			}
		}

		close( file_fd );

		/* keep_alive   Ŭ̾Ʈ ´ */
		if ( keep_alive == FALSE )
			close(self->sockfd);

		tlog_record( LOG_INFO, NULL, vhost, &request_header, &response_header );
	}

	free(buf);

	self->status = STAT_TERMINATED;
	pthread_exit(0);
}

int get_vhost_filefd( struct http_request_header_t *header, struct stat *filestat, struct vhost_t *vhost )
{
	extern struct config_t		config;

	int				i;

	if ( header->protocol > 9 && header->host != NULL )
	{
		for ( i=0; i < config.vhost_len; i++ )
		{
			if ( header->host_len == config.vhost[i].host_len && config.vhost[i].port == header->port
				&& memcmp( header->host, config.vhost[i].host, config.vhost[i].host_len ) == 0 )
			{
				vhost = &(config.vhost[i]);
				return get_read_filefd( header, config.vhost[i].document_root, filestat );
			}
		}
	}

	return get_read_filefd( header, config.document_root, filestat );
}

int get_read_filefd( struct http_request_header_t *header, char *document_root, struct stat *filestat )
{
	int				fd;
	extern struct config_t		config;

	int				i;

	snprintf( header->filename, HTTP_FILENAME_SIZE, "%s%s", document_root, header->decoded_url );

	fd = open_real_file( header->filename, filestat );
	if ( fd < 0 )
	{
		for ( i=0; i < config.directory_index_len; i++ )
		{
			snprintf( header->filename, HTTP_FILENAME_SIZE, "%s/%s", document_root, config.directory_index_file[i] );
			fd = open_real_file( header->filename, filestat );
			if ( fd > 0 )
				return fd;
		}
	}

	return fd;
}

